@@ -33,21 +33,29 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
+import time
 import sys
 from datetime import datetime
 from pathlib import Path
 from urllib.parse import unquote
 
-from gi.repository import GLib, Gio, Gtk
+from nbxmpp.namespaces import Namespace
+from nbxmpp import JID
+from nbxmpp.protocol import InvalidJid
+from gi.repository import Gio
+from gi.repository import GLib
+from gi.repository import Gtk
 
 import gajim
 from gajim.common import app
+from gajim.common import ged
 from gajim.common import configpaths
 from gajim.common import logging_helpers
 from gajim.common import exceptions
-from gajim.common import caps_cache
 from gajim.common import logger
 from gajim.common.i18n import _
+from gajim.common.contacts import LegacyContactsAPI
+from gajim.common.task_manager import TaskManager
 
 
 class GajimApplication(Gtk.Application):
@@ -55,11 +63,15 @@ class GajimApplication(Gtk.Application):
 
     def __init__(self):
         flags = (Gio.ApplicationFlags.HANDLES_COMMAND_LINE |
-                 Gio.ApplicationFlags.HANDLES_OPEN)
+                 Gio.ApplicationFlags.HANDLES_OPEN |
+                 Gio.ApplicationFlags.CAN_OVERRIDE_APP_ID)
         Gtk.Application.__init__(self,
                                  application_id='org.gajim.Gajim',
                                  flags=flags)
 
+        # required to track screensaver state
+        self.props.register_session = True
+
         self.add_main_option(
             'version',
             ord('V'),
@@ -140,21 +152,6 @@ class GajimApplication(Gtk.Application):
             GLib.OptionArg.NONE,
             _('Start a new chat'))
 
-        self.add_main_option(
-            'simulate-network-lost',
-            0,
-            GLib.OptionFlags.NONE,
-            GLib.OptionArg.NONE,
-            _('Simulate loss of connectivity'))
-
-        self.add_main_option(
-            'simulate-network-connected',
-            0,
-            GLib.OptionFlags.NONE,
-            GLib.OptionArg.NONE,
-            _('Simulate regaining connectivity'))
-
-
         self.add_main_option_entries(self._get_remaining_entry())
 
         self.connect('handle-local-options', self._handle_local_options)
@@ -164,7 +161,7 @@ class GajimApplication(Gtk.Application):
 
         self.interface = None
 
-        GLib.set_prgname('gajim')
+        GLib.set_prgname('org.gajim.Gajim')
         if GLib.get_application_name() != 'Gajim':
             GLib.set_application_name('Gajim')
 
@@ -179,51 +176,64 @@ class GajimApplication(Gtk.Application):
         option.short_name = 0
         return [option]
 
-    def _startup(self, application):
+    def _startup(self, _application):
 
         # Create and initialize Application Paths & Databases
         app.detect_dependencies()
         configpaths.create_paths()
         try:
             app.logger = logger.Logger()
-            caps_cache.initialize(app.logger)
+            app.contacts = LegacyContactsAPI()
         except exceptions.DatabaseMalformed as error:
             dlg = Gtk.MessageDialog(
-                None,
-                Gtk.DialogFlags.DESTROY_WITH_PARENT | Gtk.DialogFlags.MODAL,
-                Gtk.MessageType.ERROR,
-                Gtk.ButtonsType.OK,
-                _('Database Error'))
+                transient_for=None,
+                destroy_with_parent=True,
+                modal=True,
+                message_type=Gtk.MessageType.ERROR,
+                buttons=Gtk.ButtonsType.OK,
+                text=_('Database Error'))
             dlg.format_secondary_text(str(error))
             dlg.run()
             dlg.destroy()
             sys.exit()
 
+        from gajim.gtk.util import load_user_iconsets
+        load_user_iconsets()
+
+        from gajim.common.cert_store import CertificateStore
+        app.cert_store = CertificateStore()
+        app.task_manager = TaskManager()
+
         # Set Application Menu
         app.app = self
         from gajim.gtk.util import get_builder
         builder = get_builder('application_menu.ui')
         menubar = builder.get_object("menubar")
-        appmenu = builder.get_object("appmenu")
-        if app.prefers_app_menu():
-            self.set_app_menu(appmenu)
-        else:
-            # Add it to the menubar instead
-            menubar.prepend_submenu('Gajim', appmenu)
         self.set_menubar(menubar)
 
-    def _activate(self, application):
+    def _activate(self, _application):
         if self.interface is not None:
             self.interface.roster.window.present()
             return
+
         from gajim.gui_interface import Interface
         self.interface = Interface()
         self.interface.run(self)
         self.add_actions()
+        self._set_shortcuts()
         from gajim import gui_menu_builder
         gui_menu_builder.build_accounts_menu()
+        self.update_app_actions_state()
+
+        app.ged.register_event_handler('feature-discovered',
+                                       ged.CORE,
+                                       self._on_feature_discovered)
 
     def _open_uris(self, uris):
+        accounts = list(app.connections.keys())
+        if not accounts:
+            return
+
         for uri in uris:
             app.log('uri_handler').info('open %s', uri)
             if not uri.startswith('xmpp:'):
@@ -235,10 +245,30 @@ class GajimApplication(Gtk.Application):
             except ValueError:
                 # No query argument
                 jid, cmd = uri, 'message'
+
+            try:
+                jid = JID(jid)
+            except InvalidJid as error:
+                app.log('uri_handler').warning('Invalid JID %s: %s', uri, error)
+                continue
+
+            if cmd == 'join' and jid.getResource():
+                app.log('uri_handler').warning('Invalid MUC JID %s', uri)
+                continue
+
+            jid = str(jid)
+
             if cmd == 'join':
-                self.interface.join_gc_minimal(None, jid)
+                if len(accounts) == 1:
+                    self.activate_action(
+                        'groupchat-join',
+                        GLib.Variant('as', [accounts[0], jid]))
+                else:
+                    self.activate_action('start-chat', GLib.Variant('s', jid))
+
             elif cmd == 'roster':
                 self.activate_action('add-contact', GLib.Variant('s', jid))
+
             elif cmd.startswith('message'):
                 attributes = cmd.split(';')
                 message = None
@@ -249,15 +279,11 @@ class GajimApplication(Gtk.Application):
                         message = unquote(key.split('=')[1])
                     except Exception:
                         app.log('uri_handler').error('Invalid URI: %s', cmd)
-                accounts = list(app.connections.keys())
-                if not accounts:
-                    continue
+
                 if len(accounts) == 1:
                     app.interface.new_chat_from_jid(accounts[0], jid, message)
                 else:
-                    self.activate_action('start-chat')
-                    start_chat_window = app.interface.instances['start_chat']
-                    start_chat_window.search_entry.set_text(jid)
+                    self.activate_action('start-chat', GLib.Variant('s', jid))
 
     def do_shutdown(self, *args):
         Gtk.Application.do_shutdown(self)
@@ -268,7 +294,7 @@ class GajimApplication(Gtk.Application):
         # Commit any outstanding SQL transactions
         app.logger.commit()
 
-    def _handle_remote_options(self, application, command_line):
+    def _handle_remote_options(self, _application, command_line):
         # Parse all options that should be executed on a remote instance
         options = command_line.get_options_dict()
 
@@ -286,14 +312,6 @@ class GajimApplication(Gtk.Application):
                 self.activate_action(cmd)
                 return 0
 
-        if options.contains('simulate-network-lost'):
-            app.interface.network_status_changed(None, False)
-            return 0
-
-        if options.contains('simulate-network-connected'):
-            app.interface.network_status_changed(None, True)
-            return 0
-
         if remaining is not None:
             self._open_uris(remaining.unpack())
             return 0
@@ -302,7 +320,7 @@ class GajimApplication(Gtk.Application):
         return 0
 
     def _handle_local_options(self,
-                              application: Gtk.Application,
+                              _application: Gtk.Application,
                               options: GLib.VariantDict) -> int:
         # Parse all options that have to be executed before ::startup
         if options.contains('version'):
@@ -323,8 +341,9 @@ class GajimApplication(Gtk.Application):
 
         configpaths.init()
 
-        if app.get_win_debug_mode():
+        if app.get_debug_mode():
             # Redirect has to happen before logging init
+            self._cleanup_debug_logs()
             self._redirect_output()
             logging_helpers.init()
             logging_helpers.set_verbose()
@@ -343,12 +362,13 @@ class GajimApplication(Gtk.Application):
 
         return -1
 
-    def show_warnings(self):
+    @staticmethod
+    def show_warnings():
         import traceback
         import warnings
 
         def warn_with_traceback(message, category, filename, lineno,
-                                file=None, line=None):
+                                _file=None, line=None):
             traceback.print_stack(file=sys.stderr)
             sys.stderr.write(warnings.formatwarning(message, category,
                                                     filename, lineno, line))
@@ -359,11 +379,21 @@ class GajimApplication(Gtk.Application):
     @staticmethod
     def _redirect_output():
         debug_folder = Path(configpaths.get('DEBUG'))
-        date = datetime.today().strftime('%Y-%m-%d')
+        date = datetime.today().strftime('%d%m%Y-%H%M%S')
         filename = '%s-debug.log' % date
         fd = open(debug_folder / filename, 'a')
         sys.stderr = sys.stdout = fd
 
+    @staticmethod
+    def _cleanup_debug_logs():
+        debug_folder = Path(configpaths.get('DEBUG'))
+        debug_files = list(debug_folder.glob('*-debug.log*'))
+        now = time.time()
+        for file in debug_files:
+            # Delete everything older than 3 days
+            if file.stat().st_ctime < now - 259200:
+                file.unlink()
+
     def add_actions(self):
         ''' Build Application Actions '''
         from gajim import app_actions
@@ -376,25 +406,14 @@ class GajimApplication(Gtk.Application):
         act.connect('change-state', app_actions.on_merge_accounts)
         self.add_action(act)
 
-        act = Gio.SimpleAction.new_stateful(
-            'agent', None,
-            GLib.Variant.new_boolean(app.config.get('use_gpg_agent')))
-        act.connect('change-state', app_actions.on_use_pgp_agent)
-        self.add_action(act)
-
-        # General Actions
-
-        general_actions = [
+        actions = [
             ('quit', app_actions.on_quit),
-            ('accounts', app_actions.on_accounts),
             ('add-account', app_actions.on_add_account),
-            ('join-groupchat', app_actions.on_join_gc),
             ('manage-proxies', app_actions.on_manage_proxies),
-            ('start-chat', app_actions.on_new_chat),
-            ('bookmarks', app_actions.on_manage_bookmarks),
             ('history-manager', app_actions.on_history_manager),
             ('preferences', app_actions.on_preferences),
             ('plugins', app_actions.on_plugins),
+            ('xml-console', app_actions.on_xml_console),
             ('file-transfer', app_actions.on_file_transfers),
             ('history', app_actions.on_history),
             ('shortcuts', app_actions.on_keyboard_shortcuts),
@@ -404,24 +423,27 @@ class GajimApplication(Gtk.Application):
             ('faq', app_actions.on_faq),
             ('ipython', app_actions.toggle_ipython),
             ('show-next-pending-event', app_actions.show_next_pending_event),
+            ('start-chat', 's', app_actions.on_new_chat),
+            ('accounts', 's', app_actions.on_accounts),
+            ('add-contact', 's', app_actions.on_add_contact_jid),
+            ('copy-text', 's', app_actions.copy_text),
+            ('open-link', 'as', app_actions.open_link),
+            ('open-mail', 's', app_actions.open_mail),
+            ('create-groupchat', 's', app_actions.on_create_gc),
+            ('browse-history', 'a{sv}', app_actions.on_browse_history),
+            ('groupchat-join', 'as', app_actions.on_groupchat_join),
         ]
 
-        act = Gio.SimpleAction.new('add-contact', GLib.VariantType.new('s'))
-        act.connect("activate", app_actions.on_add_contact_jid)
-        self.add_action(act)
+        for action in actions:
+            if len(action) == 2:
+                action_name, func = action
+                variant = None
+            else:
+                action_name, variant, func = action
+                variant = GLib.VariantType.new(variant)
 
-        act = Gio.SimpleAction.new('copy-link', GLib.VariantType.new('s'))
-        act.connect("activate", app_actions.copy_link)
-        self.add_action(act)
-
-        act = Gio.SimpleAction.new('open-link', GLib.VariantType.new('as'))
-        act.connect("activate", app_actions.open_link)
-        self.add_action(act)
-
-        for action in general_actions:
-            action_name, func = action
-            act = Gio.SimpleAction.new(action_name, None)
-            act.connect("activate", func)
+            act = Gio.SimpleAction.new(action_name, variant)
+            act.connect('activate', func)
             self.add_action(act)
 
         accounts_list = sorted(app.config.get_per('accounts'))
@@ -433,31 +455,29 @@ class GajimApplication(Gtk.Application):
         else:
             self.add_account_actions(accounts_list[0])
 
-    def _get_account_actions(self, account):
+    @staticmethod
+    def _get_account_actions(account):
         from gajim import app_actions as a
 
         if account == 'Local':
-            return [
-                ('-xml-console', a.on_xml_console, 'always', 's')
-            ]
+            return []
 
         return [
+            ('-bookmarks', a.on_bookmarks, 'online', 's'),
             ('-start-single-chat', a.on_single_message, 'online', 's'),
             ('-start-chat', a.start_chat, 'online', 'as'),
-            ('-join-groupchat', a.on_join_gc, 'online', 'as'),
             ('-add-contact', a.on_add_contact, 'online', 'as'),
             ('-services', a.on_service_disco, 'online', 's'),
             ('-profile', a.on_profile, 'feature', 's'),
-            ('-xml-console', a.on_xml_console, 'always', 's'),
             ('-server-info', a.on_server_info, 'online', 's'),
             ('-archive', a.on_mam_preferences, 'feature', 's'),
+            ('-pep-config', a.on_pep_config, 'online', 's'),
             ('-sync-history', a.on_history_sync, 'online', 's'),
-            ('-privacylists', a.on_privacy_lists, 'feature', 's'),
+            ('-blocking', a.on_blocking_list, 'feature', 's'),
             ('-send-server-message', a.on_send_server_message, 'online', 's'),
             ('-set-motd', a.on_set_motd, 'online', 's'),
             ('-update-motd', a.on_update_motd, 'online', 's'),
             ('-delete-motd', a.on_delete_motd, 'online', 's'),
-            ('-activate-bookmark', a.on_activate_bookmark, 'online', 'a{sv}'),
             ('-open-event', a.on_open_event, 'always', 'a{sv}'),
             ('-import-contacts', a.on_import_contacts, 'online', 's'),
         ]
@@ -490,3 +510,66 @@ class GajimApplication(Gtk.Application):
             elif new_state and state == 'online':
                 # We go online
                 self.lookup_action(account + action_name).set_enabled(True)
+
+    def update_app_actions_state(self):
+        active_accounts = bool(app.get_connected_accounts(exclude_local=True))
+        self.lookup_action('create-groupchat').set_enabled(active_accounts)
+
+        enabled_accounts = app.contacts.get_accounts()
+        self.lookup_action('start-chat').set_enabled(enabled_accounts)
+
+    def _set_shortcuts(self):
+        shortcuts = {
+            'app.quit': ['<Primary>Q'],
+            'app.shortcuts': ['<Primary>question'],
+            'app.preferences': ['<Primary>P'],
+            'app.plugins': ['<Primary>E'],
+            'app.xml-console': ['<Primary><Shift>X'],
+            'app.file-transfer': ['<Primary>T'],
+            'app.ipython': ['<Primary><Alt>I'],
+            'app.start-chat::': ['<Primary>N'],
+            'app.create-groupchat::': ['<Primary>G'],
+            'win.show-roster': ['<Primary>R'],
+            'win.show-offline': ['<Primary>O'],
+            'win.show-active': ['<Primary>Y'],
+            'win.change-nickname': ['<Control><Shift>n'],
+            'win.change-subject': ['<Alt>t'],
+            'win.escape': ['Escape'],
+            'win.browse-history': ['<Control>h'],
+            'win.send-file': ['<Control>f'],
+            'win.show-contact-info': ['<Control>i'],
+            'win.show-emoji-chooser': ['<Alt>m'],
+            'win.clear-chat': ['<Control>l'],
+            'win.delete-line': ['<Control>u'],
+            'win.close-tab': ['<Control>w'],
+            'win.move-tab-up': ['<Control><Shift>Page_Up'],
+            'win.move-tab-down': ['<Control><Shift>Page_Down'],
+            'win.switch-next-tab': ['<Control>Page_Down'],
+            'win.switch-prev-tab': ['<Control>Page_Up'],
+            'win.switch-next-unread-tab-right': ['<Control>Tab'],
+            'win.switch-next-unread-tab-left': ['<Control>ISO_Left_Tab'],
+            'win.switch-tab-1': ['<Alt>1', '<Alt>KP_1'],
+            'win.switch-tab-2': ['<Alt>2', '<Alt>KP_2'],
+            'win.switch-tab-3': ['<Alt>3', '<Alt>KP_3'],
+            'win.switch-tab-4': ['<Alt>4', '<Alt>KP_4'],
+            'win.switch-tab-5': ['<Alt>5', '<Alt>KP_5'],
+            'win.switch-tab-6': ['<Alt>6', '<Alt>KP_6'],
+            'win.switch-tab-7': ['<Alt>7', '<Alt>KP_7'],
+            'win.switch-tab-8': ['<Alt>8', '<Alt>KP_8'],
+            'win.switch-tab-9': ['<Alt>9', '<Alt>KP_9'],
+            'win.copy-text': ['<Control><Shift>c'],
+        }
+
+        for action, accels in shortcuts.items():
+            self.set_accels_for_action(action, accels)
+
+    def _on_feature_discovered(self, event):
+        if event.feature == Namespace.VCARD:
+            action = '%s-profile' % event.account
+            self.lookup_action(action).set_enabled(True)
+        elif event.feature == Namespace.MAM_2:
+            action = '%s-archive' % event.account
+            self.lookup_action(action).set_enabled(True)
+        elif event.feature == Namespace.BLOCKING:
+            action = '%s-blocking' % event.account
+            self.lookup_action(action).set_enabled(True)
