@@ -28,16 +28,22 @@ This module allows to access the on-disk database of logs
 import os
 import sys
 import time
+import math
 import datetime
 import calendar
 import json
 import logging
 import sqlite3 as sqlite
 from collections import namedtuple
-from gzip import GzipFile
-from io import BytesIO
+
 from gi.repository import GLib
 
+from nbxmpp.protocol import Iq
+from nbxmpp.protocol import JID
+from nbxmpp.structs import DiscoInfo
+from nbxmpp.structs import CommonError
+from nbxmpp.modules.discovery import parse_disco_info
+
 from gajim.common import exceptions
 from gajim.common import app
 from gajim.common import configpaths
@@ -69,9 +75,11 @@ LOGS_SQL_STATEMENT = '''
             kind INTEGER,
             show INTEGER,
             message TEXT,
+            error TEXT,
             subject TEXT,
             additional_data TEXT,
             stanza_id TEXT,
+            message_id TEXT,
             encryption TEXT,
             encryption_state TEXT,
             marker INTEGER
@@ -85,7 +93,8 @@ LOGS_SQL_STATEMENT = '''
     );
     CREATE INDEX idx_logs_jid_id_time ON logs (jid_id, time DESC);
     CREATE INDEX idx_logs_stanza_id ON logs (stanza_id);
-    PRAGMA user_version=2;
+    CREATE INDEX idx_logs_message_id ON logs (message_id);
+    PRAGMA user_version=5;
     '''
 
 CACHE_SQL_STATEMENT = '''
@@ -96,11 +105,13 @@ CACHE_SQL_STATEMENT = '''
     CREATE TABLE caps_cache (
             hash_method TEXT,
             hash TEXT,
-            data BLOB,
-            last_seen INTEGER);
-    CREATE TABLE rooms_last_message_time(
-            jid_id INTEGER PRIMARY KEY UNIQUE,
-            time INTEGER
+            data TEXT,
+            last_seen INTEGER
+    );
+    CREATE TABLE last_seen_disco_info(
+            jid TEXT PRIMARY KEY UNIQUE,
+            disco_info TEXT,
+            last_seen INTEGER
     );
     CREATE TABLE roster_entry(
             account_jid_id INTEGER,
@@ -117,12 +128,61 @@ CACHE_SQL_STATEMENT = '''
             group_name TEXT,
             PRIMARY KEY (account_jid_id, jid_id, group_name)
     );
-    PRAGMA user_version=1;
+    CREATE TABLE muc_avatars(
+            jid TEXT PRIMARY KEY UNIQUE,
+            avatar_sha TEXT
+    );
+    PRAGMA user_version=5;
     '''
 
 log = logging.getLogger('gajim.c.logger')
 
 
+def timeit(func):
+    def func_wrapper(self, *args, **kwargs):
+        start = time.time()
+        result = func(self, *args, **kwargs)
+        exec_time = (time.time() - start) * 1e3
+        level = 30 if exec_time > 50 else 10
+        log.log(level, 'Execution time for %s: %s ms',
+                func.__name__, math.ceil(exec_time))
+        return result
+    return func_wrapper
+
+def _convert_disco_info(disco_info):
+    return parse_disco_info(Iq(node=disco_info))
+
+def _adapt_disco_info(disco_info):
+    return str(disco_info.stanza)
+
+def _convert_common_error(common_error):
+    return CommonError.from_string(common_error)
+
+def _adapt_common_error(common_error):
+    return common_error.serialize()
+
+def _convert_marker(marker):
+    return 'received' if marker == 0 else 'displayed'
+
+def _jid_adapter(jid):
+    return str(jid)
+
+def _jid_converter(jid):
+    return JID(jid.decode())
+
+
+sqlite.register_converter('jid', _jid_converter)
+sqlite.register_adapter(JID, _jid_adapter)
+
+sqlite.register_converter('disco_info', _convert_disco_info)
+sqlite.register_adapter(DiscoInfo, _adapt_disco_info)
+
+sqlite.register_converter('common_error', _convert_common_error)
+sqlite.register_adapter(CommonError, _adapt_common_error)
+
+sqlite.register_converter('marker', _convert_marker)
+
+
 class Logger:
     def __init__(self):
         self._jid_ids = {}
@@ -132,10 +192,18 @@ class Logger:
         self._log_db_path = configpaths.get('LOG_DB')
         self._cache_db_path = configpaths.get('CACHE_DB')
 
+        self._entity_caps_cache = {}
+        self._disco_info_cache = {}
+        self._muc_avatar_sha_cache = {}
+
         self._create_databases()
         self._migrate_databases()
         self._connect_databases()
         self._get_jid_ids_from_db()
+        self._fill_disco_info_cache()
+        self._fill_muc_avatar_sha_cache()
+        self._clean_caps_table()
+        self._load_caps_data()
 
     def _create_databases(self):
         if os.path.isdir(self._log_db_path):
@@ -197,6 +265,7 @@ class Logger:
             log.exception('Error')
             sys.exit()
 
+    @timeit
     def _migrate_logs(self, con):
         if self._get_user_version(con) == 0:
             # All migrations from 0.16.9 until 1.0.0
@@ -228,6 +297,28 @@ class Logger:
             ]
             self._execute_multiple(con, statements)
 
+        if self._get_user_version(con) < 3:
+            statements = [
+                'ALTER TABLE logs ADD COLUMN "message_id" TEXT',
+                'PRAGMA user_version=3'
+            ]
+            self._execute_multiple(con, statements)
+
+        if self._get_user_version(con) < 4:
+            statements = [
+                'ALTER TABLE logs ADD COLUMN "error" TEXT',
+                'PRAGMA user_version=4'
+            ]
+            self._execute_multiple(con, statements)
+
+        if self._get_user_version(con) < 5:
+            statements = [
+                'CREATE INDEX idx_logs_message_id ON logs (message_id)',
+                'PRAGMA user_version=5'
+            ]
+            self._execute_multiple(con, statements)
+
+    @timeit
     def _migrate_cache(self, con):
         if self._get_user_version(con) == 0:
             # All migrations from 0.16.9 until 1.0.0
@@ -238,12 +329,43 @@ class Logger:
             self._execute_multiple(con, statements)
 
         if self._get_user_version(con) < 2:
-            pass
+            statements = [
+                'DROP TABLE IF EXISTS caps_cache',
+                'CREATE TABLE caps_cache (hash_method TEXT, hash TEXT, data TEXT, last_seen INTEGER)',
+                'PRAGMA user_version=2'
+                ]
+            self._execute_multiple(con, statements)
+
+        if self._get_user_version(con) < 3:
+            statements = [
+                '''CREATE TABLE last_seen_disco_info(
+                    jid TEXT PRIMARY KEY UNIQUE,
+                    disco_info TEXT,
+                    last_seen INTEGER)''',
+                'PRAGMA user_version=3'
+                ]
+            self._execute_multiple(con, statements)
+
+        if self._get_user_version(con) < 4:
+            statements = [
+                '''CREATE TABLE muc_avatars(
+                   jid TEXT PRIMARY KEY UNIQUE,
+                   avatar_sha TEXT)''',
+                'PRAGMA user_version=4'
+                ]
+            self._execute_multiple(con, statements)
+
+        if self._get_user_version(con) < 5:
+            statements = [
+                'DELETE FROM caps_cache',
+                'PRAGMA user_version=5'
+                ]
+            self._execute_multiple(con, statements)
 
     @staticmethod
     def _execute_multiple(con, statements):
         """
-        Execute mutliple statements with the option to fail on duplicates
+        Execute multiple statements with the option to fail on duplicates
         but still continue
         """
         for sql in statements:
@@ -282,8 +404,10 @@ class Logger:
         app.ged.raise_event(event, None, str(error))
 
     def _connect_databases(self):
-        self._con = self._connect(
-            self._log_db_path, timeout=20.0, isolation_level='IMMEDIATE')
+        self._con = self._connect(self._log_db_path,
+                                  timeout=20.0,
+                                  isolation_level='IMMEDIATE',
+                                  detect_types=sqlite.PARSE_COLNAMES)
 
         self._con.row_factory = self.namedtuple_factory
 
@@ -300,6 +424,7 @@ class Logger:
             self._con.close()
             sys.exit()
 
+    @timeit
     def _set_synchronous(self, sync):
         try:
             if sync:
@@ -325,6 +450,7 @@ class Logger:
     def _like(search_str):
         return '%{}%'.format(search_str)
 
+    @timeit
     def commit(self):
         try:
             self._con.commit()
@@ -338,6 +464,7 @@ class Logger:
             return
         self._commit_timout_id = GLib.timeout_add(500, self.commit)
 
+    @timeit
     def simple_commit(self, sql_to_commit):
         """
         Helper to commit
@@ -345,6 +472,7 @@ class Logger:
         self._con.execute(sql_to_commit)
         self._timeout_commit()
 
+    @timeit
     def _get_jid_ids_from_db(self):
         """
         Load all jid/jid_id tuples into a dict for faster access
@@ -363,7 +491,7 @@ class Logger:
         If jid is gajim@conf/nkour it's likely a pm one, how we know gajim@conf
         is not a normal guy and nkour is not his resource?  we ask if gajim@conf
         is already in jids (with type room jid) this fails if user disables
-        logging for room and only enables for pm (so higly unlikely) and if we
+        logging for room and only enables for pm (so highly unlikely) and if we
         fail we do not go chaos (user will see the first pm as if it was message
         in room's public chat) and after that all okay
         """
@@ -398,10 +526,11 @@ class Logger:
             return [user['jid'] for user in family]
         return [jid]
 
-    def get_account_id(self, account):
+    def get_account_id(self, account, type_=JIDConstant.NORMAL_TYPE):
         jid = app.get_jid_from_account(account)
-        return self.get_jid_id(jid, type_=JIDConstant.NORMAL_TYPE)
+        return self.get_jid_id(jid, type_=type_)
 
+    @timeit
     def get_jid_id(self, jid, kind=None, type_=None):
         """
         Get the jid id from a jid.
@@ -442,29 +571,6 @@ class Logger:
         self._timeout_commit()
         return lastrowid
 
-    def convert_kind_values_to_db_api_values(self, kind):
-        """
-        Convert from string style to constant ints for db
-        """
-        if kind == 'status':
-            kind_col = KindConstant.STATUS
-        elif kind == 'gcstatus':
-            kind_col = KindConstant.GCSTATUS
-        elif kind == 'gc_msg':
-            kind_col = KindConstant.GC_MSG
-        elif kind == 'single_msg_recv':
-            kind_col = KindConstant.SINGLE_MSG_RECV
-        elif kind == 'single_msg_sent':
-            kind_col = KindConstant.SINGLE_MSG_SENT
-        elif kind == 'chat_msg_recv':
-            kind_col = KindConstant.CHAT_MSG_RECV
-        elif kind == 'chat_msg_sent':
-            kind_col = KindConstant.CHAT_MSG_SENT
-        elif kind == 'error':
-            kind_col = KindConstant.ERROR
-
-        return kind_col
-
     def convert_show_values_to_db_api_values(self, show):
         """
         Convert from string style to constant ints for db
@@ -581,6 +687,7 @@ class Logger:
         if sub == SubscriptionConstant.BOTH:
             return 'both'
 
+    @timeit
     def insert_unread_events(self, message_id, jid_id):
         """
         Add unread message with id: message_id
@@ -590,6 +697,7 @@ class Logger:
         self._con.execute(sql, (message_id, jid_id))
         self._timeout_commit()
 
+    @timeit
     def set_read_messages(self, message_ids):
         """
         Mark all messages with ids in message_ids as read
@@ -598,6 +706,7 @@ class Logger:
         sql = 'DELETE FROM unread_messages WHERE message_id IN (%s)' % ids
         self.simple_commit(sql)
 
+    @timeit
     def set_shown_unread_msgs(self, msg_log_id):
         """
         Mark unread message as shown un GUI
@@ -606,6 +715,7 @@ class Logger:
                 msg_log_id
         self.simple_commit(sql)
 
+    @timeit
     def reset_shown_unread_messages(self):
         """
         Set shown field to False in unread_messages table
@@ -613,6 +723,7 @@ class Logger:
         sql = 'UPDATE unread_messages SET shown = 0'
         self.simple_commit(sql)
 
+    @timeit
     def get_unread_msgs(self):
         """
         Get all unread messages
@@ -644,12 +755,33 @@ class Logger:
             all_messages.append((result, shown))
         return all_messages
 
+    def load_groupchat_messages(self, account, jid):
+        account_id = self.get_account_id(account, type_=JIDConstant.ROOM_TYPE)
+
+        sql = '''
+            SELECT time, contact_name, message, additional_data, message_id
+            FROM logs NATURAL JOIN jids WHERE jid = ?
+            AND account_id = ? AND kind = ?
+            ORDER BY time DESC, log_line_id DESC LIMIT ?'''
+
+        try:
+            messages = self._con.execute(
+                sql, (jid, account_id, KindConstant.GC_MSG, 50)).fetchall()
+        except sqlite.DatabaseError:
+            self.dispatch('DB_ERROR',
+                          exceptions.DatabaseMalformed(self._log_db_path))
+            return []
+
+        messages.reverse()
+        return messages
+
+    @timeit
     def get_last_conversation_lines(self, account, jid, pending):
         """
         Get recent messages
 
         Pending messages are already in queue to be printed when the
-        ChatControl is opened, so we dont want to request those messages.
+        ChatControl is opened, so we don’t want to request those messages.
         How many messages are requested depends on the 'restore_lines'
         config value. How far back in time messages are requested depends on
         _get_timeout().
@@ -658,7 +790,7 @@ class Logger:
 
         :param jid:     The jid from which we request the conversation lines
 
-        :param pending: How many messages are currently pending so we dont
+        :param pending: How many messages are currently pending so we don’t
                         request those messages
 
         returns a list of namedtuples
@@ -675,13 +807,18 @@ class Logger:
                           KindConstant.ERROR])
 
         jids = self._get_family_jids(account, jid)
+        account_id = self.get_account_id(account)
 
         sql = '''
-            SELECT time, kind, message, subject, additional_data
-            FROM logs NATURAL JOIN jids WHERE jid IN ({jids}) AND
-            kind IN ({kinds}) AND time > get_timeout()
+            SELECT time, kind, message, error as "error [common_error]",
+                   subject, additional_data, marker as "marker [marker]",
+                   message_id
+            FROM logs NATURAL JOIN jids WHERE jid IN ({jids})
+            AND account_id = {account_id} AND kind IN ({kinds})
+            AND time > get_timeout()
             ORDER BY time DESC, log_line_id DESC LIMIT ? OFFSET ?
             '''.format(jids=', '.join('?' * len(jids)),
+                       account_id=account_id,
                        kinds=', '.join(kinds))
 
         try:
@@ -705,6 +842,7 @@ class Logger:
         start_of_day = int(time.mktime(local_time))
         return start_of_day
 
+    @timeit
     def get_conversation_for_date(self, account, jid, date):
         """
         Load the complete conversation with a given jid on a specific date
@@ -736,12 +874,13 @@ class Logger:
                                       (date.timestamp(),
                                       (date + delta).timestamp())).fetchall()
 
+    @timeit
     def search_log(self, account, jid, query, date=None):
         """
         Search the conversation log for messages containing the `query` string.
 
         The search can either span the complete log for the given
-        `account` and `jid` or be restriced to a single day by
+        `account` and `jid` or be restricted to a single day by
         specifying `date`.
 
         :param account: The account
@@ -771,12 +910,13 @@ class Logger:
                additional_data, log_line_id
         FROM logs NATURAL JOIN jids WHERE jid IN ({jids})
         AND message LIKE like(?) {date_search}
-        ORDER BY time, log_line_id
+        ORDER BY time DESC, log_line_id
         '''.format(jids=', '.join('?' * len(jids)),
                    date_search=between if date else '')
 
         return self._con.execute(sql, tuple(jids) + (query,)).fetchall()
 
+    @timeit
     def get_days_with_logs(self, account, jid, year, month):
         """
         Request the days in a month where we received messages
@@ -817,6 +957,7 @@ class Logger:
                                       (date.timestamp(),
                                       (date + delta).timestamp())).fetchall()
 
+    @timeit
     def get_last_date_that_has_logs(self, account, jid):
         """
         Get the timestamp of the last message we received for the jid.
@@ -843,6 +984,7 @@ class Logger:
         # attributes set to None because of the MAX() function
         return self._con.execute(sql, tuple(jids)).fetchone().time
 
+    @timeit
     def get_first_date_that_has_logs(self, account, jid):
         """
         Get the timestamp of the first message we received for the jid.
@@ -869,6 +1011,7 @@ class Logger:
         # attributes set to None because of the MIN() function
         return self._con.execute(sql, tuple(jids)).fetchone().time
 
+    @timeit
     def get_date_has_logs(self, account, jid, date):
         """
         Get single timestamp of a message we received for the jid
@@ -900,45 +1043,7 @@ class Logger:
         return self._con.execute(
             sql, tuple(jids) + (start, end)).fetchone()
 
-    def get_room_last_message_time(self, account, jid):
-        """
-        Get the timestamp of the last message we received in a room.
-
-        :param account: The account
-
-        :param jid:     The jid for which we request the last timestamp
-
-        returns a timestamp or None
-        """
-        sql = '''
-            SELECT time FROM rooms_last_message_time
-            NATURAL JOIN jids WHERE jid = ?
-            '''
-
-        row = self._con.execute(sql, (jid,)).fetchone()
-        if not row:
-            return self.get_last_date_that_has_logs(account, jid)
-        return row.time
-
-    def set_room_last_message_time(self, jid, timestamp):
-        """
-        Set the timestamp of the last message we received in a room.
-
-        :param jid:         The jid
-
-        :param timestamp:   The timestamp in epoch
-
-        """
-
-        jid_id = self.get_jid_id(jid, type_=JIDConstant.ROOM_TYPE)
-        sql = '''REPLACE INTO rooms_last_message_time
-                 VALUES (:jid_id, COALESCE(
-                 (SELECT time FROM rooms_last_message_time
-                  WHERE jid_id = :jid_id AND time >= :time), :time))'''
-
-        self._con.execute(sql, {"jid_id": jid_id, "time": timestamp})
-        self._timeout_commit()
-
+    @timeit
     def save_transport_type(self, jid, type_):
         """
         Save the type of the transport in DB
@@ -960,6 +1065,7 @@ class Logger:
         self._con.execute(sql, (jid, type_id))
         self._timeout_commit()
 
+    @timeit
     def get_transports_type(self):
         """
         Return all the type of the transports in DB
@@ -973,105 +1079,51 @@ class Logger:
                     result.type)
         return answer
 
-    # A longer note here:
-    # The database contains a blob field. Pysqlite seems to need special care for
-    # such fields.
-    # When storing, we need to convert string into buffer object (1).
-    # When retrieving, we need to convert it back to a string to decompress it.
-    # (2)
-    # GzipFile needs a file-like object, StringIO emulates file for plain strings
-    def iter_caps_data(self):
-        """
-        Iterate over caps cache data stored in the database
+    @timeit
+    def _load_caps_data(self):
+        '''
+        Load caps cache data
+        '''
+        rows = self._con.execute(
+            'SELECT hash_method, hash, data as "data [disco_info]" '
+            'FROM caps_cache')
 
-        The iterator values are pairs of (node, ver, ext, identities, features):
-        identities == {'category':'foo', 'type':'bar', 'name':'boo'},
-        features being a list of feature namespaces.
-        """
-        # get data from table
-        # the data field contains binary object (gzipped data), this is a hack
-        # to get that data without trying to convert it to unicode
-        try:
-            rows = self._con.execute('SELECT hash_method, hash, data FROM caps_cache;')
-        except sqlite.OperationalError:
-            # might happen when there's no caps_cache table yet
-            # -- there's no data to read anyway then
-            return
-
-        # list of corrupted entries that will be removed
-        to_be_removed = []
         for row in rows:
-            # for each row: unpack the data field
-            # (format: (category, type, name, category, type, name, ...
-            #   ..., 'FEAT', feature1, feature2, ...).join(' '))
-            # NOTE: if there's a need to do more gzip, put that to a function
-            try:
-                data = GzipFile(fileobj=BytesIO(row.data)).read().decode('utf-8').split('\0')
-            except IOError:
-                # This data is corrupted. It probably contains non-ascii chars
-                to_be_removed.append((row.hash_method, row.hash))
-                continue
-            i = 0
-            identities = list()
-            features = list()
-            while i < (len(data) - 3) and data[i] != 'FEAT':
-                category = data[i]
-                type_ = data[i + 1]
-                lang = data[i + 2]
-                name = data[i + 3]
-                identities.append({'category': category, 'type': type_,
-                                   'xml:lang': lang, 'name': name})
-                i += 4
-            i += 1
-            while i < len(data):
-                features.append(data[i])
-                i += 1
+            self._entity_caps_cache[(row.hash_method, row.hash)] = row.data
 
-            # yield the row
-            yield row.hash_method, row.hash, identities, features
-        for hash_method, hash_ in to_be_removed:
-            sql = '''DELETE FROM caps_cache WHERE hash_method = "%s" AND
-                    hash = "%s"''' % (hash_method, hash_)
-            self.simple_commit(sql)
+    @timeit
+    def add_caps_entry(self, jid, hash_method, hash_, caps_data):
+        self._entity_caps_cache[(hash_method, hash_)] = caps_data
+
+        self._disco_info_cache[jid] = caps_data
 
-    def add_caps_entry(self, hash_method, hash_, identities, features):
-        data = []
-        for identity in identities:
-            # there is no FEAT category
-            if identity['category'] == 'FEAT':
-                return
-            data.extend((identity.get('category'), identity.get('type', ''),
-                    identity.get('xml:lang', ''), identity.get('name', '')))
-        data.append('FEAT')
-        data.extend(features)
-        data = '\0'.join(data)
-        # if there's a need to do more gzip, put that to a function
-        string = BytesIO()
-        gzip = GzipFile(fileobj=string, mode='w')
-        gzip.write(data.encode('utf-8'))
-        gzip.close()
-        data = string.getvalue()
         self._con.execute('''
-                INSERT INTO caps_cache ( hash_method, hash, data, last_seen )
-                VALUES (?, ?, ?, ?);
-                ''', (hash_method, hash_, memoryview(data), int(time.time())))
-        # (1) -- note above
+                INSERT INTO caps_cache (hash_method, hash, data, last_seen)
+                VALUES (?, ?, ?, ?)
+                ''', (hash_method, hash_, caps_data, int(time.time())))
         self._timeout_commit()
 
-    def update_caps_time(self, method, hash_):
-        sql = '''UPDATE caps_cache SET last_seen = %d
-                WHERE hash_method = "%s" and hash = "%s"''' % \
-                (int(time.time()), method, hash_)
-        self.simple_commit(sql)
+    def get_caps_entry(self, hash_method, hash_):
+        return self._entity_caps_cache.get((hash_method, hash_))
 
-    def clean_caps_table(self):
+    @timeit
+    def update_caps_time(self, method, hash_):
+        sql = '''UPDATE caps_cache SET last_seen = ?
+                 WHERE hash_method = ? and hash = ?'''
+        self._con.execute(sql, (int(time.time()), method, hash_))
+        self._timeout_commit()
+
+    @timeit
+    def _clean_caps_table(self):
         """
         Remove caps which was not seen for 3 months
         """
-        sql = '''DELETE FROM caps_cache WHERE last_seen < %d''' % \
-                int(time.time() - 3*30*24*3600)
-        self.simple_commit(sql)
+        timestamp = int(time.time()) - 3 * 30 * 24 * 3600
+        self._con.execute('DELETE FROM caps_cache WHERE last_seen < ?',
+                          (timestamp,))
+        self._timeout_commit()
 
+    @timeit
     def replace_roster(self, account_name, roster_version, roster):
         """
         Replace current roster in DB by a new one
@@ -1081,7 +1133,7 @@ class Logger:
         roster is the new version.
         """
         # First we must reset roster_version value to ensure that the server
-        # sends back all the roster at the next connexion if the replacement
+        # sends back all the roster at the next connection if the replacement
         # didn't work properly.
         app.config.set_per('accounts', account_name, 'roster_version', '')
 
@@ -1105,6 +1157,7 @@ class Logger:
         app.config.set_per('accounts', account_name, 'roster_version',
             roster_version)
 
+    @timeit
     def del_contact(self, account_jid, jid):
         """
         Remove jid from account_jid roster
@@ -1122,6 +1175,7 @@ class Logger:
                 (account_jid_id, jid_id))
         self._timeout_commit()
 
+    @timeit
     def add_or_update_contact(self, account_jid, jid, name, sub, ask, groups,
                               commit=True):
         """
@@ -1161,6 +1215,7 @@ class Logger:
         if commit:
             self._timeout_commit()
 
+    @timeit
     def get_roster(self, account_jid):
         """
         Return the accound_jid roster in NonBlockingRoster format
@@ -1168,7 +1223,7 @@ class Logger:
         data = {}
         account_jid_id = self.get_jid_id(account_jid, type_=JIDConstant.NORMAL_TYPE)
 
-        # First we fill data with roster_entry informations
+        # First we fill data with roster_entry information
         rows = self._con.execute('''
                 SELECT j.jid, re.jid_id, re.name, re.subscription, re.ask, re.avatar_sha
                 FROM roster_entry re, jids j
@@ -1205,6 +1260,7 @@ class Logger:
 
         return data
 
+    @timeit
     def remove_roster(self, account_jid):
         """
         Remove the roster of an account
@@ -1226,41 +1282,55 @@ class Logger:
         self._con.executescript(sql)
         self._timeout_commit()
 
-    def search_for_duplicate(self, account, jid, timestamp, msg):
+    @timeit
+    def deduplicate_muc_message(self, account, jid, resource,
+                                timestamp, message_id):
         """
         Check if a message is already in the `logs` table
 
         :param account:     The account
 
-        :param jid:         The jid as string
+        :param jid:         The muc jid as string
+
+        :param resource:    The resource
 
         :param timestamp:   The timestamp in UTC epoch
 
-        :param msg:         The message text
+        :param message_id:  The message-id
         """
 
-        # Add 10 seconds around the timestamp
-        start_time = timestamp - 30
-        end_time = timestamp + 30
+        # Add 60 seconds around the timestamp
+        start_time = timestamp - 60
+        end_time = timestamp + 60
 
         account_id = self.get_account_id(account)
-        log.debug('start: %s, end: %s, jid: %s, message: %s',
-                  start_time, end_time, jid, msg)
+        log.debug('Search for MUC duplicate')
+        log.debug('start: %s, end: %s, jid: %s, resource: %s, message-id: %s',
+                  start_time, end_time, jid, resource, message_id)
 
         sql = '''
             SELECT * FROM logs
-            NATURAL JOIN jids WHERE jid = ? AND message = ? AND account_id = ?
-            AND time BETWEEN ? AND ?
+            NATURAL JOIN jids WHERE
+            jid = ? AND
+            contact_name = ? AND
+            message_id = ? AND
+            account_id = ? AND
+            time BETWEEN ? AND ?
             '''
 
-        result = self._con.execute(
-            sql, (jid, msg, account_id, start_time, end_time)).fetchone()
+        result = self._con.execute(sql, (jid,
+                                         resource,
+                                         message_id,
+                                         account_id,
+                                         start_time,
+                                         end_time)).fetchone()
 
         if result is not None:
-            log.debug('Message already in DB')
+            log.debug('Found duplicate')
             return True
         return False
 
+    @timeit
     def find_stanza_id(self, account, archive_jid, stanza_id, origin_id=None,
                        groupchat=False):
         """
@@ -1299,10 +1369,14 @@ class Logger:
             # Stanza ID is only unique within a specific archive.
             # So a Stanza ID could be repeated in different MUCs, so we
             # filter also for the archive JID which is the bare MUC jid.
+
+            # Use Unary-"+" operator for "jid_id", otherwise the
+            # idx_logs_jid_id_time index is used instead of the much better
+            # idx_logs_stanza_id index
             sql = '''
                 SELECT stanza_id FROM logs
                 WHERE stanza_id IN ({values})
-                AND jid_id = ? AND account_id = ? LIMIT 1
+                AND +jid_id = ? AND account_id = ? LIMIT 1
                 '''.format(values=', '.join('?' * len(ids)))
             result = self._con.execute(
                 sql, tuple(ids) + (archive_id, account_id)).fetchone()
@@ -1333,6 +1407,7 @@ class Logger:
         """
         return self.get_jid_id(jid, kind, type_)
 
+    @timeit
     def insert_into_logs(self, account, jid, time_, kind,
                          unread=True, **kwargs):
         """
@@ -1381,6 +1456,77 @@ class Logger:
 
         return lastrowid
 
+    @timeit
+    def set_message_error(self, account_jid, jid, message_id, error):
+        """
+        Update the corresponding message with the error
+
+        :param account_jid: The jid of the account
+
+        :param jid:         The jid that belongs to the avatar
+
+        :param message_id:  The id of the message
+
+        :param error:       The error stanza as string
+
+        """
+
+        account_id = self.get_jid_id(account_jid)
+        try:
+            jid_id = self.get_jid_id(str(jid))
+        except ValueError:
+            # Unknown JID
+            return
+
+        sql = '''
+            UPDATE logs SET error = ?
+            WHERE account_id = ? AND jid_id = ? AND message_id = ?
+            '''
+        self._con.execute(sql, (error, account_id, jid_id, message_id))
+        self._timeout_commit()
+
+    @timeit
+    def set_marker(self, account_jid, jid, message_id, state):
+        """
+        Update the marker state of the corresponding message
+
+        :param account_jid: The jid of the account
+
+        :param jid:         The jid that belongs to the avatar
+
+        :param message_id:  The id of the message
+
+        :param state:       The state, 'received' or 'displayed'
+
+        """
+        if state not in ('received', 'displayed'):
+            raise ValueError('Invalid marker state')
+
+        account_id = self.get_jid_id(account_jid)
+        try:
+            jid_id = self.get_jid_id(str(jid))
+        except ValueError:
+            # Unknown JID
+            return
+
+        state = 0 if state == 'received' else 1
+
+        sql = '''
+            UPDATE logs SET marker = ?
+            WHERE account_id = ? AND jid_id = ? AND message_id = ?
+            '''
+        self._con.execute(sql, (state, account_id, jid_id, message_id))
+        self._timeout_commit()
+
+    @timeit
+    def _fill_muc_avatar_sha_cache(self):
+        sql = '''SELECT jid, avatar_sha FROM muc_avatars'''
+        rows = self._con.execute(sql).fetchall()
+        for row in rows:
+            self._muc_avatar_sha_cache[row.jid] = row.avatar_sha
+        log.info('%d Avatar SHA entries loaded', len(rows))
+
+    @timeit
     def set_avatar_sha(self, account_jid, jid, sha=None):
         """
         Set the avatar sha of a jid on an account
@@ -1403,6 +1549,41 @@ class Logger:
         self._con.execute(sql, (sha, account_jid_id, jid_id))
         self._timeout_commit()
 
+    @timeit
+    def set_muc_avatar_sha(self, jid, sha=None):
+        """
+        Set the avatar sha of a MUC
+
+        :param jid:         The MUC jid that belongs to the avatar
+
+        :param sha:         The sha of the avatar
+
+        """
+
+        sql = '''INSERT INTO muc_avatars (jid, avatar_sha)
+                 VALUES (?, ?)'''
+
+        try:
+            self._con.execute(sql, (jid, sha))
+        except sqlite.IntegrityError:
+            sql = 'UPDATE muc_avatars SET avatar_sha = ? WHERE jid = ?'
+            self._con.execute(sql, (sha, jid))
+
+        self._muc_avatar_sha_cache[jid] = sha
+
+        self._timeout_commit()
+
+    def get_muc_avatar_sha(self, jid):
+        """
+        Get the avatar sha of a MUC
+
+        :param jid:         The MUC jid that belongs to the avatar
+
+        """
+
+        return self._muc_avatar_sha_cache.get(jid)
+
+    @timeit
     def get_archive_infos(self, jid):
         """
         Get the archive infos
@@ -1414,6 +1595,7 @@ class Logger:
         sql = '''SELECT * FROM last_archive_message WHERE jid_id = ?'''
         return self._con.execute(sql, (jid_id,)).fetchone()
 
+    @timeit
     def set_archive_infos(self, jid, **kwargs):
         """
         Set archive infos
@@ -1447,9 +1629,93 @@ class Logger:
                 kwargs.get('sync_threshold', None)
             ))
         else:
+            for key, value in list(kwargs.items()):
+                if value is None:
+                    del kwargs[key]
+
             args = ' = ?, '.join(kwargs.keys()) + ' = ?'
             sql = '''UPDATE last_archive_message SET {}
                      WHERE jid_id = ?'''.format(args)
             self._con.execute(sql, tuple(kwargs.values()) + (jid_id,))
-        log.info('Save archive infos: %s', kwargs)
+        log.info('Set message archive info: %s %s', jid, kwargs)
+        self._timeout_commit()
+
+    @timeit
+    def reset_archive_infos(self, jid):
+        """
+        Set archive infos
+
+        :param jid:                     The jid of the archive
+
+        """
+        jid_id = self.get_jid_id(jid)
+        sql = '''UPDATE last_archive_message
+                 SET last_mam_id = NULL, oldest_mam_timestamp = NULL,
+                 last_muc_timestamp = NULL
+                 WHERE jid_id = ?'''
+        self._con.execute(sql, (jid_id,))
+        log.info('Reset message archive info: %s', jid)
+        self._timeout_commit()
+
+    @timeit
+    def _fill_disco_info_cache(self):
+        sql = '''SELECT disco_info as "disco_info [disco_info]",
+                 jid, last_seen FROM
+                 last_seen_disco_info'''
+        rows = self._con.execute(sql).fetchall()
+        for row in rows:
+            disco_info = row.disco_info._replace(timestamp=row.last_seen)
+            self._disco_info_cache[row.jid] = disco_info
+        log.info('%d DiscoInfo entries loaded', len(rows))
+
+    def get_last_disco_info(self, jid, max_age=0):
+        """
+        Get last disco info from jid
+
+        :param jid:         The jid
+
+        :param max_age:     max age in seconds of the DiscoInfo record
+
+        """
+
+        disco_info = self._disco_info_cache.get(jid)
+        if disco_info is not None:
+            max_timestamp = time.time() - max_age if max_age else 0
+            if max_timestamp > disco_info.timestamp:
+                return None
+        return disco_info
+
+    @timeit
+    def set_last_disco_info(self, jid, disco_info, cache_only=False):
+        """
+        Get last disco info from jid
+
+        :param jid:          The jid
+
+        :param disco_info:   A DiscoInfo object
+
+        """
+
+        log.info('Save disco info from %s', jid)
+
+        if cache_only:
+            self._disco_info_cache[jid] = disco_info
+            return
+
+        disco_exists = self.get_last_disco_info(jid) is not None
+        if disco_exists:
+            sql = '''UPDATE last_seen_disco_info SET
+                     disco_info = ?, last_seen = ?
+                     WHERE jid = ?'''
+
+            self._con.execute(sql, (disco_info, disco_info.timestamp, str(jid)))
+
+        else:
+            sql = '''INSERT INTO last_seen_disco_info
+                     (jid, disco_info, last_seen)
+                     VALUES (?, ?, ?)'''
+
+            self._con.execute(sql, (str(jid), disco_info, disco_info.timestamp))
+
+        self._disco_info_cache[jid] = disco_info
         self._timeout_commit()
