@@ -22,26 +22,67 @@ import signal
 import platform
 from ctypes import CDLL, byref, create_string_buffer
 from ctypes.util import find_library
-from distutils.version import LooseVersion as V
+from packaging.version import Version as V
 
-# Install _() in namespace
 from gajim.common import i18n
 
-_MIN_NBXMPP_VER = "0.6.9"
-_MIN_GTK_VER = "3.22.0"
+_MIN_NBXMPP_VER = '1.0.1'
+_MIN_GTK_VER = '3.22.27'
+_MIN_CAIRO_VER = '1.16.0'
+_MIN_PYGOBJECT_VER = '3.32.0'
+_MIN_GLIB_VER = '2.60.0'
+
+
+def check_version(dep_name, current_ver, min_ver):
+    if V(current_ver) < V(min_ver):
+        sys.exit('Gajim needs %s >= %s (found %s) to run. '
+                 'Quitting...' % (dep_name, min_ver, current_ver))
 
 
 def _check_required_deps():
+    error_message = 'Gajim needs %s to run. Quitting… (Error: %s)'
+
     try:
         import nbxmpp
-    except ImportError:
-        print('Gajim needs python-nbxmpp to run. Quitting…')
-        sys.exit(1)
+    except ImportError as error:
+        sys.exit(error_message % ('python-nbxmpp', error))
 
-    if V(nbxmpp.__version__) < V(_MIN_NBXMPP_VER):
-        print('Gajim needs python-nbxmpp >= %s to run. '
-              'Quitting...' % _MIN_NBXMPP_VER)
-        sys.exit(1)
+    try:
+        import gi
+    except ImportError as error:
+        sys.exit(error_message % ('pygobject', error))
+
+    try:
+        gi.require_versions({'GLib': '2.0',
+                             'Gio': '2.0',
+                             'Gtk': '3.0',
+                             'GObject': '2.0',
+                             'Pango': '1.0',
+                             'Soup': '2.4'})
+    except ValueError as error:
+        sys.exit('Missing dependency: %s' % error)
+
+    try:
+        import cairo
+    except ImportError as error:
+        sys.exit(error_message % ('python-cairo', error))
+
+    from gi.repository import Gtk
+    gtk_ver = '%s.%s.%s' % (Gtk.get_major_version(),
+                            Gtk.get_minor_version(),
+                            Gtk.get_micro_version())
+
+    from gi.repository import GLib
+    glib_ver = '.'.join(map(str, [GLib.MAJOR_VERSION,
+                                  GLib.MINOR_VERSION,
+                                  GLib.MICRO_VERSION]))
+
+    check_version('python-nbxmpp', nbxmpp.__version__, _MIN_NBXMPP_VER)
+    check_version('pygobject', gi.__version__, _MIN_PYGOBJECT_VER)
+    check_version('libcairo', cairo.cairo_version_string(), _MIN_CAIRO_VER)
+    check_version('python-cairo', cairo.version, _MIN_CAIRO_VER)
+    check_version('gtk3', gtk_ver, _MIN_GTK_VER)
+    check_version('glib', glib_ver, _MIN_GLIB_VER)
 
 
 def _init_gui(gui):
@@ -49,36 +90,25 @@ def _init_gui(gui):
         _init_gtk()
 
 
+def _disable_csd():
+    if sys.platform != 'win32':
+        return
+
+    if 'GTK_OSD' in os.environ:
+        # Respect user settings
+        return
+
+    os.environ['GTK_CSD'] = '0'
+
+
 def _init_gtk():
-    import gi
-    gi.require_version('GLib', '2.0')
-    gi.require_version('Gio', '2.0')
-    gi.require_version('Gtk', '3.0')
-    gi.require_version('Gdk', '3.0')
-    gi.require_version('GObject', '2.0')
-    gi.require_version('Pango', '1.0')
-
-    from gi.repository import Gtk
-    gtk_ver = '%s.%s.%s' % (Gtk.get_major_version(),
-                            Gtk.get_minor_version(),
-                            Gtk.get_micro_version())
-    if V(gtk_ver) < V(_MIN_GTK_VER):
-        print('Gajim needs GTK+ >= %s to run. '
-              'Quitting...' % _MIN_GTK_VER)
-        sys.exit(1)
-
-    from gajim import gtkexcepthook
-    gtkexcepthook.init()
+    from gajim.gtk import exception
+    exception.init()
 
     i18n.initialize_direction_mark()
 
     from gajim.application import GajimApplication
 
-    # Fix bug in 3.24.1 with flickering tooltips
-    if sys.platform in ('darwin', 'win32'):
-        settings = Gtk.Settings.get_default()
-        settings.set_property('gtk-cursor-theme-size', 16)
-
     application = GajimApplication()
     _install_sginal_handlers(application)
     application.run(sys.argv)
@@ -109,15 +139,16 @@ def _install_sginal_handlers(application):
     # ^C exits the application normally
     signal.signal(signal.SIGINT, sigint_cb)
     signal.signal(signal.SIGTERM, sigint_cb)
-    if os.name != 'nt':
+    if sys.platform != 'win32':
         signal.signal(signal.SIGPIPE, signal.SIG_DFL)
 
 
 def main():
     if sys.platform != 'win32':
         if os.geteuid() == 0:
-            sys.exit("You must not launch gajim as root, it is insecure.")
+            sys.exit('You must not launch gajim as root, it is insecure.')
 
     _check_required_deps()
     _set_proc_title()
+    _disable_csd()
     _init_gui('GTK')
