@@ -23,8 +23,7 @@
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
 import os
-import base64
-import mimetypes
+import logging
 
 from gi.repository import GLib
 from gi.repository import Gio
@@ -33,8 +32,10 @@ from gajim.common import app
 from gajim.common import helpers
 from gajim.gtk.add_contact import AddNewContactWindow
 from gajim.common import ged
-from gajim.common.connection_handlers_events import MessageOutgoingEvent
-from gajim.common.connection_handlers_events import GcMessageOutgoingEvent
+from gajim.common.structs import OutgoingMessage
+
+
+log = logging.getLogger('gajim.remote_control')
 
 
 def get_dbus_struct(obj):
@@ -97,7 +98,7 @@ class Server:
 
         result = getattr(self, method_name)(*args)
 
-        # out_args is atleast (signature1). We therefore always wrap the result
+        # out_args is at least (signature1). We therefore always wrap the result
         # as a tuple. Refer to https://bugzilla.gnome.org/show_bug.cgi?id=765603
         result = (result, )
 
@@ -129,10 +130,6 @@ class GajimRemote(Server):
                 <arg name='account' type='s' />
                 <arg direction='out' type='b' />
             </method>
-            <method name='change_avatar'>
-                <arg name='picture' type='s' />
-                <arg name='account' type='s' />
-            </method>
             <method name='change_status'>
                 <arg name='status' type='s' />
                 <arg name='message' type='s' />
@@ -152,7 +149,6 @@ class GajimRemote(Server):
             </method>
             <method name='join_room'>
                 <arg name='room_jid' type='s' />
-                <arg name='nick' type='s' />
                 <arg name='password' type='s' />
                 <arg name='account' type='s' />
             </method>
@@ -191,7 +187,6 @@ class GajimRemote(Server):
             <method name='send_chat_message'>
                 <arg name='jid' type='s' />
                 <arg name='message' type='s' />
-                <arg name='keyID' type='s' />
                 <arg name='account' type='s' />
                 <arg direction='out' type='b' />
             </method>
@@ -211,7 +206,6 @@ class GajimRemote(Server):
                 <arg name='jid' type='s' />
                 <arg name='subject' type='s' />
                 <arg name='message' type='s' />
-                <arg name='keyID' type='s' />
                 <arg name='account' type='s' />
                 <arg direction='out' type='b' />
             </method>
@@ -226,7 +220,7 @@ class GajimRemote(Server):
             <method name='show_next_pending_event' />
             <method name='show_roster' />
             <method name='start_chat'>
-                <arg name='account' type='s' />
+                <arg name='jid' type='s' />
                 <arg direction='out' type='b' />
             </method>
             <method name='toggle_ipython' />
@@ -264,9 +258,6 @@ class GajimRemote(Server):
             <signal name='NewMessage'>
                 <arg type='av' />
             </signal>
-            <signal name='OsInfo'>
-                <arg type='av' />
-            </signal>
             <signal name='Roster'>
                 <arg type='av' />
             </signal>
@@ -296,8 +287,6 @@ class GajimRemote(Server):
         super().__init__(self.con, '/org/gajim/dbus/RemoteObject')
         self.first_show = True
 
-        app.ged.register_event_handler('version-result-received', ged.POSTGUI,
-            self.on_os_info)
         app.ged.register_event_handler('time-result-received', ged.POSTGUI,
             self.on_time)
         app.ged.register_event_handler('roster-info', ged.POSTGUI,
@@ -340,14 +329,8 @@ class GajimRemote(Server):
             chatstate = obj.chatstate
         except AttributeError:
             chatstate = ''
-        self.raise_signal('MessageSent', (obj.conn.name, [
-            obj.jid, obj.message, obj.keyID, chatstate]))
-
-    def on_os_info(self, obj):
-        self.raise_signal('OsInfo', (obj.conn.name, [obj.jid.getStripped(),
-                                                     obj.jid.getResource(),
-                                                     obj.client_info,
-                                                     obj.os_info]))
+        self.raise_signal('MessageSent', (obj.account, [
+            obj.jid, obj.message, chatstate]))
 
     def on_time(self, obj):
         self.raise_signal('EntityTime', (obj.conn.name, [obj.jid.getStripped(),
@@ -368,16 +351,16 @@ class GajimRemote(Server):
         else:
             return
         self.raise_signal(event, (obj.conn.name, [obj.jid, obj.show,
-                obj.status, obj.resource, obj.prio, obj.keyID, obj.timestamp,
-                obj.contact_nickname]))
+                obj.status, obj.resource, obj.prio, obj.timestamp]))
 
     def on_subscribe_presence_received(self, obj):
         self.raise_signal('Subscribe', (obj.conn.name, [obj.jid, obj.status,
             obj.user_nick]))
 
-    def on_subscribed_presence_received(self, obj):
-        self.raise_signal('Subscribed', (obj.conn.name, [obj.jid,
-            obj.resource]))
+    def on_subscribed_presence_received(self, event):
+        self.raise_signal('Subscribed', (event.account,
+                                         [event.jid.getBare(),
+                                          event.jid.getResource()]))
 
     def on_unsubscribed_presence_received(self, obj):
         self.raise_signal('Unsubscribed', (obj.conn.name, obj.jid))
@@ -387,25 +370,29 @@ class GajimRemote(Server):
             # event has not been handled at GUI level
             return
         self.raise_signal('GCMessage', (obj.conn.name, [obj.fjid, obj.msgtxt,
-            obj.timestamp, obj.delayed, obj.xhtml_msgtxt, obj.status_code,
-            obj.displaymarking, obj.captcha_form, obj.needs_highlight]))
+            obj.properties.timestamp, obj.delayed,
+            obj.displaymarking, obj.needs_highlight]))
 
     def _nec_decrypted_message_received(self, obj):
+        event_type = obj.properties.type.value
+        if obj.properties.is_muc_pm:
+            event_type = 'pm'
         self.raise_signal('NewMessage', (
-            obj.conn.name, [obj.fjid, obj.msgtxt, obj.timestamp,
-            obj.encrypted, obj.mtype, obj.subject,
-            obj.msg_log_id, obj.user_nick, obj.xhtml, obj.form_node]))
+            obj.conn.name, [obj.fjid, obj.msgtxt, obj.properties.timestamp,
+            event_type, obj.properties.subject,
+            obj.msg_log_id, obj.properties.nickname]))
 
-    def on_our_status(self, obj):
-        self.raise_signal('AccountPresence', (obj.show, obj.conn.name))
+    def on_our_status(self, event):
+        self.raise_signal('AccountPresence', (event.show, event.account))
 
     def on_account_created(self, obj):
         self.raise_signal('NewAccount', (obj.conn.name, obj.account_info))
 
     def on_vcard_received(self, obj):
-        self.raise_signal('VcardInfo', (obj.conn.name, obj.vcard_dict))
+        self.raise_signal('VcardInfo', (obj.account, obj.vcard_dict))
 
     def raise_signal(self, event_name, data):
+        log.info('Send event %s', event_name)
         self.con.emit_signal(None,
                              '/org/gajim/dbus/RemoteObject',
                              'org.gajim.dbus.RemoteInterface',
@@ -421,8 +408,7 @@ class GajimRemote(Server):
             # If user did not ask for account, returns the global status
             return helpers.get_global_show()
         # return show for the given account
-        index = app.connections[account].connected
-        return app.SHOW_LIST[index]
+        return app.connections[account].status
 
     def get_status_message(self, account):
         """
@@ -430,7 +416,7 @@ class GajimRemote(Server):
         """
         if not account:
             # If user did not ask for account, returns the global status
-            return str(helpers.get_global_status())
+            return str(helpers.get_global_status_message())
         # return show for the given account
         status = app.connections[account].status
         return status
@@ -447,7 +433,7 @@ class GajimRemote(Server):
         if not account and len(accounts) == 1:
             account = accounts[0]
         if account:
-            if app.connections[account].connected > 1:  # account is connected
+            if app.account_is_available(account):  # account is connected
                 connected_account = account
                 contact = app.contacts.get_contact_with_highest_priority(
                     account, jid)
@@ -455,7 +441,7 @@ class GajimRemote(Server):
             for account_ in accounts:
                 contact = app.contacts.get_contact_with_highest_priority(
                     account, jid)
-                if contact and app.connections[account_].connected > 1:
+                if contact and app.account_is_available(account_):
                     # account is connected
                     connected_account = account_
                     break
@@ -476,14 +462,14 @@ class GajimRemote(Server):
         if not account and len(accounts) == 1:
             account = accounts[0]
         if account:
-            if app.connections[account].connected > 1 and \
+            if app.account_is_available(account) and \
                     room_jid in app.gc_connected[account] and \
                     app.gc_connected[account][room_jid]:
                 # account and groupchat are connected
                 connected_account = account
         else:
             for account_ in accounts:
-                if app.connections[account_].connected > 1 and \
+                if app.account_is_available(account_) and \
                         room_jid in app.gc_connected[account_] and \
                         app.gc_connected[account_][room_jid]:
                     # account and groupchat are connected
@@ -512,7 +498,6 @@ class GajimRemote(Server):
     def _send_message(self,
                       jid,
                       message,
-                      keyID,
                       account,
                       type_='chat',
                       subject=None):
@@ -522,66 +507,64 @@ class GajimRemote(Server):
         """
         if not jid or not message:
             return False
-        if not keyID:
-            keyID = ''
 
-        connected_account = self._get_account_and_contact(account, jid)[0]
-        if connected_account:
-            connection = app.connections[connected_account]
-            sessions = connection.get_sessions(jid)
-            if not sessions:
-                connection.make_new_session(jid)
-            ctrl = app.interface.msg_win_mgr.search_control(
-                jid, connected_account)
-            if ctrl:
-                ctrl.send_message(message)
-            else:
-                app.nec.push_outgoing_event(
-                    MessageOutgoingEvent(
-                        None,
-                        account=connected_account,
-                        jid=jid,
-                        message=message,
-                        keyID=keyID,
-                        type_=type_,
-                        control=ctrl))
+        connected_account, contact = self._get_account_and_contact(account, jid)
+        if not connected_account or contact is None:
+            return False
 
-            return True
-        return False
+        connection = app.connections[connected_account]
+        ctrl = app.interface.msg_win_mgr.search_control(
+            jid, connected_account)
+        if ctrl:
+            ctrl.send_message(message)
+        else:
+            message_ = OutgoingMessage(account=connected_account,
+                                       contact=contact,
+                                       message=message,
+                                       type_=type_,
+                                       subject=subject,
+                                       control=ctrl)
+            connection.send_message(message_)
+        return True
 
-    def send_chat_message(self, jid, message, keyID, account):
+    def send_chat_message(self, jid, message, account):
         """
-        Send chat 'message' to 'jid', using account (optional) 'account'. If keyID
-        is specified, encrypt the message with the pgp key
+        Send chat 'message' to 'jid', using account (optional) 'account'.
         """
         jid = self._get_real_jid(jid, account)
-        return self._send_message(jid, message, keyID, account)
+        return self._send_message(jid, message, account)
 
-    def send_single_message(self, jid, subject, message, keyID, account):
+    def send_single_message(self, jid, subject, message, account):
         """
-        Send single 'message' to 'jid', using account (optional) 'account'. If
-        keyID is specified, encrypt the message with the pgp key
+        Send single 'message' to 'jid', using account (optional) 'account'.
         """
         jid = self._get_real_jid(jid, account)
-        return self._send_message(jid, message, keyID, account, 'normal',
-                                  subject)
+        return self._send_message(jid, message, account, 'normal', subject)
 
     def send_groupchat_message(self, room_jid, message, account):
         """
-        Send 'message' to groupchat 'room_jid', using account (optional) 'account'
+        Send 'message' to groupchat 'room_jid',
+        using account (optional) 'account'
         """
         if not room_jid or not message:
             return False
         connected_account = self._get_account_for_groupchat(account, room_jid)
-        if connected_account:
-            app.nec.push_outgoing_event(
-                GcMessageOutgoingEvent(
-                    None,
-                    account=connected_account,
-                    jid=room_jid,
-                    message=message))
-            return True
-        return False
+        if not connected_account:
+            return False
+
+        contact = app.contacts.get_groupchat_contact(connected_account,
+                                                     room_jid)
+        if contact is None:
+            return False
+
+        message_ = OutgoingMessage(account=connected_account,
+                                   contact=contact,
+                                   message=message,
+                                   type_='groupchat')
+        con = app.connections[connected_account]
+        con.send_message(message_)
+        return True
+
 
     def open_chat(self, jid, account, message):
         """
@@ -607,20 +590,20 @@ class GajimRemote(Server):
         connected_account = None
         first_connected_acct = None
         for acct in accounts:
-            if app.connections[acct].connected > 1:  # account is  online
+            if app.account_is_available(acct):  # account is  online
                 contact = app.contacts.get_first_contact_from_jid(acct, jid)
                 if app.interface.msg_win_mgr.has_window(jid, acct):
                     connected_account = acct
                     break
                 # jid is in roster
-                elif contact:
+                if contact:
                     minimized_control = \
                         jid in app.interface.minimized_controls[acct]
                     connected_account = acct
                     break
                 # we send the message to jid not in roster, because account is
                 # specified, or there is only one account
-                elif account:
+                if account:
                     connected_account = acct
                 elif first_connected_acct is None:
                     first_connected_acct = acct
@@ -648,14 +631,13 @@ class GajimRemote(Server):
         change_status(status, message, account). Account is optional - if not
         specified status is changed for all accounts
         """
-        if status not in ('offline', 'online', 'chat', 'away', 'xa', 'dnd',
-                          'invisible'):
+        if status not in ('offline', 'online', 'chat', 'away', 'xa', 'dnd'):
             status = ''
         if account:
             if not status:
                 if account not in app.connections:
                     return False
-                status = app.SHOW_LIST[app.connections[account].connected]
+                status = app.connections[account].status
             GLib.idle_add(app.interface.roster.send_status, account, status,
                           message)
         else:
@@ -669,7 +651,7 @@ class GajimRemote(Server):
                 else:
                     if acc not in app.connections:
                         continue
-                    status_ = app.SHOW_LIST[app.connections[acc].connected]
+                    status_ = app.connections[acc].status
                 GLib.idle_add(app.interface.roster.send_status, acc, status_,
                               message)
         return False
@@ -681,20 +663,20 @@ class GajimRemote(Server):
         """
         if account:
             app.config.set_per('accounts', account, 'priority', prio)
-            show = app.SHOW_LIST[app.connections[account].connected]
-            status = app.connections[account].status
+            show = app.connections[account].status
+            status = app.connections[account].status_message
             GLib.idle_add(app.connections[account].change_status, show, status)
         else:
             # account not specified, so change prio of all accounts
             for acc in app.contacts.get_accounts():
-                if not app.account_is_connected(acc):
+                if not app.account_is_available(acc):
                     continue
                 if not app.config.get_per('accounts', acc,
                                           'sync_with_global_status'):
                     continue
                 app.config.set_per('accounts', acc, 'priority', prio)
-                show = app.SHOW_LIST[app.connections[acc].connected]
-                status = app.connections[acc].status
+                show = app.connections[acc].status
+                status = app.connections[acc].status_message
                 GLib.idle_add(app.connections[acc].change_status, show, status)
 
     def show_next_pending_event(self):
@@ -726,11 +708,10 @@ class GajimRemote(Server):
         if account in app.connections:
             # account is valid
             con = app.connections[account]
-            index = con.connected
-            result['status'] = app.SHOW_LIST[index]
+            result['status'] = con.status
             result['name'] = con.name
             result['jid'] = app.get_jid_from_account(con.name)
-            result['message'] = con.status
+            result['message'] = con.status_message
             result['priority'] = str(con.priority)
             result['resource'] = app.config.get_per('accounts', con.name,
                                                     'resource')
@@ -807,16 +788,15 @@ class GajimRemote(Server):
 
     def add_contact(self, jid, account):
         if account:
-            if account in app.connections and \
-                    app.connections[account].connected > 1:
+            if app.account_is_available(account):
                 # if given account is active, use it
-                AddNewContactWindow(account=account, jid=jid)
+                AddNewContactWindow(account=account, contact_jid=jid)
             else:
                 # wrong account
                 return False
         else:
             # if account is not given, show account combobox
-            AddNewContactWindow(account=None, jid=jid)
+            AddNewContactWindow(account=None, contact_jid=jid)
         return True
 
     def remove_contact(self, jid, account):
@@ -887,14 +867,7 @@ class GajimRemote(Server):
         contact_dict['name'] = GLib.Variant('s', name)
         contact_dict['show'] = GLib.Variant('s', prim_contact.show)
         contact_dict['jid'] = GLib.Variant('s', prim_contact.jid)
-        if prim_contact.keyID:
-            keyID = None
-            if len(prim_contact.keyID) == 8:
-                keyID = prim_contact.keyID
-            elif len(prim_contact.keyID) == 16:
-                keyID = prim_contact.keyID[8:]
-            if keyID:
-                contact_dict['openpgp'] = GLib.Variant('s', keyID)
+
         resources = GLib.VariantBuilder(GLib.VariantType('a(sis)'))
         for contact in contacts:
             resource_props = (contact.resource, int(contact.priority),
@@ -915,11 +888,8 @@ class GajimRemote(Server):
                 unread -= 1
         return str(unread)
 
-    def start_chat(self, account):
-        if not account:
-            # error is shown in gajim-remote check_arguments(..)
-            return False
-        app.app.activate_action('start-chat')
+    def start_chat(self, jid=''):
+        app.app.activate_action('start-chat', GLib.Variant('s', jid))
         return True
 
     def send_xml(self, xml, account):
@@ -929,41 +899,12 @@ class GajimRemote(Server):
             for acc in app.contacts.get_accounts():
                 app.connections[acc].send_stanza(str(xml))
 
-    def change_avatar(self, picture, account):
-        filesize = os.path.getsize(picture)
-        invalid_file = False
-        if os.path.isfile(picture):
-            stat = os.stat(picture)
-            if stat[6] == 0:
-                invalid_file = True
-        else:
-            invalid_file = True
-        if not invalid_file and filesize < 16384:
-            sha = app.interface.save_avatar(picture, publish=True)
-            if sha is None:
-                return
-            app.config.set_per('accounts', self.name, 'avatar_sha', sha)
-            data = app.interface.get_avatar(sha, publish=True)
-            avatar = base64.b64encode(data).decode('utf-8')
-            avatar_mime_type = mimetypes.guess_type(picture)[0]
-            vcard = {}
-            vcard['PHOTO'] = {'BINVAL': avatar}
-            if avatar_mime_type:
-                vcard['PHOTO']['TYPE'] = avatar_mime_type
-            if account:
-                app.connections[account].get_module('VCardTemp').send_vcard(
-                    vcard, sha)
-            else:
-                for acc in app.connections:
-                    app.connections[acc].get_module('VCardTemp').send_vcard(
-                        vcard, sha)
-
-    def join_room(self, room_jid, nick, password, account):
+    def join_room(self, room_jid, password, account):
         if not account:
             # get the first connected account
             accounts = app.connections.keys()
             for acct in accounts:
-                if app.account_is_connected(acct):
+                if app.account_is_available(acct):
                     if not app.connections[acct].is_zeroconf:
                         account = acct
                         break
@@ -974,10 +915,9 @@ class GajimRemote(Server):
             # zeroconf not support groupchats
             return
 
-        if not nick:
-            app.interface.join_gc_minimal(account, room_jid)
-        else:
-            app.interface.join_gc_room(account, room_jid, nick, password)
+        app.interface.show_or_join_groupchat(account,
+                                             room_jid,
+                                             password=password)
 
     def Introspect(self):
         return self.__doc__
