@@ -21,7 +21,6 @@ import string
 import random
 import itertools
 
-from gajim import notify
 from gajim.common import helpers
 from gajim.common import events
 from gajim.common import app
@@ -29,7 +28,8 @@ from gajim.common import contacts
 from gajim.common import ged
 from gajim.common.helpers import AdditionalDataDict
 from gajim.common.const import KindConstant
-from gajim.gtk.single_message import SingleMessageWindow
+from gajim.gtk.util import get_show_in_roster
+from gajim.gtk.util import get_show_in_systray
 
 
 class ChatControlSession:
@@ -70,8 +70,10 @@ class ChatControlSession:
                                      self.jid.getStripped())
 
     def get_to(self):
-        to = str(self.jid)
-        return app.get_jid_without_resource(to) + '/' + self.resource
+        bare_jid = self.jid.getBare()
+        if not self.resource:
+            return bare_jid
+        return bare_jid + '/' + self.resource
 
     def _nec_decrypted_message_received(self, obj):
         """
@@ -80,7 +82,7 @@ class ChatControlSession:
         if obj.session != self:
             return
 
-        if obj.muc_pm:
+        if obj.properties.is_muc_pm:
             contact = app.contacts.get_gc_contact(
                 self.conn.name, obj.jid, obj.resource)
         else:
@@ -97,48 +99,30 @@ class ChatControlSession:
                 if self.control.resource:
                     self.control.change_resource(self.resource)
 
-        if obj.mtype == 'chat':
-            if not obj.msgtxt:
-                return
+        if not obj.msgtxt:
+            return
 
-            log_type = KindConstant.CHAT_MSG_RECV
-            if obj.forwarded and obj.sent:
-                log_type = KindConstant.CHAT_MSG_SENT
-        else:
-            log_type = KindConstant.SINGLE_MSG_RECV
-            if obj.forwarded and obj.sent:
-                log_type = KindConstant.SINGLE_MSG_SENT
-
-        treat_as = app.config.get('treat_incoming_messages')
-        if treat_as:
-            obj.mtype = treat_as
-        pm = False
-        if obj.muc_pm or (obj.gc_control and obj.resource):
-            # It's a Private message
-            pm = True
-            obj.mtype = 'pm'
+        log_type = KindConstant.CHAT_MSG_RECV
+        if obj.properties.is_sent_carbon:
+            log_type = KindConstant.CHAT_MSG_SENT
 
         if self.is_loggable() and obj.msgtxt:
-            if obj.xhtml and app.config.get('log_xhtml_messages'):
-                msg_to_log = obj.xhtml
-            else:
-                msg_to_log = obj.msgtxt
-
             jid = obj.fjid
-            if not pm:
+            if not obj.properties.is_muc_pm:
                 jid = obj.jid
 
             obj.msg_log_id = app.logger.insert_into_logs(
-                self.conn.name, jid, obj.timestamp, log_type,
-                message=msg_to_log,
-                subject=obj.subject,
+                self.conn.name,
+                jid,
+                obj.properties.timestamp,
+                log_type,
+                message=obj.msgtxt,
+                subject=obj.properties.subject,
                 additional_data=obj.additional_data,
-                stanza_id=obj.unique_id)
+                stanza_id=obj.unique_id,
+                message_id=obj.properties.id)
 
-        self.conn.get_module('MAM').save_archive_id(
-            None, obj.stanza_id, obj.timestamp)
-
-        if obj.muc_pm and not obj.gc_control:
+        if obj.properties.is_muc_pm and not obj.gc_control:
             # This is a carbon of a PM from a MUC we are not currently
             # joined. We log it silently without notification.
             return True
@@ -146,11 +130,6 @@ class ChatControlSession:
         if not obj.msgtxt: # empty message text
             return True
 
-        if app.config.get_per('accounts', self.conn.name,
-        'ignore_unknown_contacts') and not app.contacts.get_contacts(
-        self.conn.name, obj.jid) and not pm:
-            return True
-
         if not self.control:
             ctrl = app.interface.msg_win_mgr.search_control(obj.jid,
                 obj.conn.name, obj.resource)
@@ -163,15 +142,9 @@ class ChatControlSession:
                 else:
                     self.control.contact = contact
 
-        if not pm:
+        if not obj.properties.is_muc_pm:
             self.roster_message2(obj)
 
-        if app.interface.remote_ctrl:
-            app.interface.remote_ctrl.raise_signal('NewMessage', (
-                self.conn.name, [obj.fjid, obj.msgtxt, obj.timestamp,
-                obj.encrypted, obj.mtype, obj.subject,
-                obj.msg_log_id, obj.user_nick, obj.xhtml, obj.form_node]))
-
     def roster_message2(self, obj):
         """
         Display the message or show notification in the roster
@@ -210,7 +183,7 @@ class ChatControlSession:
         if not contact:
             # contact is not in roster
             contact = app.interface.roster.add_to_not_in_the_roster(
-                obj.conn.name, jid, obj.user_nick)
+                obj.conn.name, jid, obj.properties.nickname)
 
         if not self.control:
             ctrl = app.interface.msg_win_mgr.search_control(obj.jid,
@@ -226,16 +199,12 @@ class ChatControlSession:
         event_t = events.ChatEvent
         event_type = 'message_received'
 
-        if obj.mtype == 'normal':
-            event_t = events.NormalEvent
-            event_type = 'single_message_received'
-
-        if self.control and obj.mtype != 'normal':
+        if self.control:
             # We have a ChatControl open
             obj.show_in_roster = False
             obj.show_in_systray = False
             do_event = False
-        elif obj.forwarded and obj.sent:
+        elif obj.properties.is_sent_carbon:
             # Its a Carbon Copied Message we sent
             obj.show_in_roster = False
             obj.show_in_systray = False
@@ -249,30 +218,32 @@ class ChatControlSession:
             do_event = False
         else:
             # Everything else
-            obj.show_in_roster = notify.get_show_in_roster(event_type,
-                self.conn.name, contact.jid, self)
-            obj.show_in_systray = notify.get_show_in_systray(event_type,
-                self.conn.name, contact.jid)
-            if obj.mtype == 'normal' and obj.popup:
-                do_event = False
-            else:
-                do_event = True
+            obj.show_in_roster = get_show_in_roster(event_type, self)
+            obj.show_in_systray = get_show_in_systray(event_type, contact.jid)
+            do_event = True
         if do_event:
-            event = event_t(obj.msgtxt, obj.subject, obj.mtype, obj.timestamp,
-                obj.encrypted, obj.resource, obj.msg_log_id,
-                correct_id=(obj.id_, obj.correct_id), xhtml=obj.xhtml,
-                session=self, form_node=obj.form_node,
+            kind = obj.properties.type.value
+            event = event_t(
+                obj.msgtxt,
+                obj.properties.subject,
+                kind,
+                obj.properties.timestamp,
+                obj.resource,
+                obj.msg_log_id,
+                correct_id=obj.correct_id,
+                message_id=obj.properties.id,
+                session=self,
                 displaymarking=obj.displaymarking,
-                sent_forwarded=obj.forwarded and obj.sent,
+                sent_forwarded=obj.properties.is_sent_carbon,
                 show_in_roster=obj.show_in_roster,
                 show_in_systray=obj.show_in_systray,
                 additional_data=obj.additional_data)
 
             app.events.add_event(self.conn.name, fjid, event)
 
-    def roster_message(self, jid, msg, tim, encrypted=False, msg_type='',
-    subject=None, resource='', msg_log_id=None, user_nick='', xhtml=None,
-    form_node=None, displaymarking=None, additional_data=None):
+    def roster_message(self, jid, msg, tim, msg_type='',
+    subject=None, resource='', msg_log_id=None, user_nick='',
+    displaymarking=None, additional_data=None):
         """
         Display the message or show notification in the roster
         """
@@ -327,22 +298,19 @@ class ChatControlSession:
 
         popup = helpers.allow_popup_window(self.conn.name)
 
-        if msg_type == 'normal' and popup: # it's single message to be autopopuped
-            SingleMessageWindow(self.conn.name, contact.jid,
-                    action='receive', from_whom=jid, subject=subject, message=msg,
-                    resource=resource, session=self, form_node=form_node)
-            return
-
         # We print if window is opened and it's not a single message
-        if self.control and msg_type != 'normal':
+        if self.control:
             typ = ''
 
             if msg_type == 'error':
                 typ = 'error'
 
-            self.control.print_conversation(msg, typ, tim=tim, encrypted=encrypted,
-                    subject=subject, xhtml=xhtml, displaymarking=displaymarking,
-                    additional_data=additional_data)
+            self.control.add_message(msg,
+                                     typ,
+                                     tim=tim,
+                                     subject=subject,
+                                     displaymarking=displaymarking,
+                                     additional_data=additional_data)
 
             if msg_log_id:
                 app.logger.set_read_messages([msg_log_id])
@@ -353,17 +321,11 @@ class ChatControlSession:
         event_t = events.ChatEvent
         event_type = 'message_received'
 
-        if msg_type == 'normal':
-            event_t = events.NormalEvent
-            event_type = 'single_message_received'
+        show_in_roster = get_show_in_roster(event_type, self)
+        show_in_systray = get_show_in_systray(event_type, contact.jid)
 
-        show_in_roster = notify.get_show_in_roster(event_type, self.conn.name,
-                contact.jid, self)
-        show_in_systray = notify.get_show_in_systray(event_type, self.conn.name,
-                contact.jid)
-
-        event = event_t(msg, subject, msg_type, tim, encrypted, resource,
-            msg_log_id, xhtml=xhtml, session=self, form_node=form_node,
+        event = event_t(msg, subject, msg_type, tim, resource,
+            msg_log_id, session=self,
             displaymarking=displaymarking, sent_forwarded=False,
             show_in_roster=show_in_roster, show_in_systray=show_in_systray,
             additional_data=additional_data)
